/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

--*/

#ifndef _PHIDGETPLUGIN_H_
#define _PHIDGETPLUGIN_H_

//////////////////////////////////////////////////////////////////////////
// architecture specific definitions

#if !defined(_WINDOWS) && !defined(_UNIX) && !defined(_MACOS)
#error Please define the OS
#endif

// declare Windows-specific callback format (__stdcall)
#if defined(_WINDOWS)
#define CALLBACK __stdcall
#else
#define CALLBACK
#endif // _WINDOWS

// catch all those different definitions of deprecated directives...
#define deprecated(x)
// catch cygwin not knowing the __int64 type
#ifdef __CYGWIN__
typedef long long __int64;
#endif


//////////////////////////////////////////////////////////////////////////
// includes: C Standard Header Files

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>

// includes: Phidget Library
// catch multiple includes of Windows' phidget21.h
#ifndef _PHIDGET21_H_
#define _PHIDGET21_H_
#include <phidget21.h>
#endif //_PHIDGET21_H_

// includes: Squeak Virtual Machine
#include "sqVirtualMachine.h"

// includes: Phidget Plugin types
#include "PhidgetPluginTypes.h"

// includes: Phidget Classes
#include "Phidget.h"
#include "PhidgetRemote.h"
#include "PhidgetInterfaceKit.h"
#include "PhidgetServo.h"
#include "PhidgetRFID.h"
#include "PhidgetAccelerometer.h"
#include "PhidgetMotorControl.h"
#include "PhidgetAdvancedServo.h"
#include "PhidgetTextLCD.h"

// includes: Phidget Component Classes
#include "PhidgetComponent.h"
#include "PhidgetCompAnalogSensor.h"
#include "PhidgetCompDigitalInput.h"
#include "PhidgetCompDigitalOutput.h"
#include "PhidgetCompServoMotor.h"
#include "PhidgetCompRFID.h"
#include "PhidgetCompSimpleLED.h"
#include "PhidgetCompAxis.h"
#include "PhidgetCompMotor.h"
#include "PhidgetCompAdvancedServoMotor.h"
#include "PhidgetCompTextDisplay.h"
#include "PhidgetCompTextLCD.h"

// includes: Phidget Plugin Debugging
#include "PhidgetDebug.h"

// includes: Phidget Manager
#include "PhidgetManager.h"


//////////////////////////////////////////////////////////////////////////
// global variables declaration

// Squeak VM interpreter proxy to access VM in the plugin and vice versa.
// This variable is created in the by the VMMaker generated PhidgetPlugin.c
// file.
extern struct VirtualMachine* interpreterProxy;
// Last phidget error code. See the phidget library for more information
// about phidget error codes.
extern int g_iLastError;

//////////////////////////////////////////////////////////////////////////
// Plugin Macros Definition

//
// Routine Description:
//
//     Retrieve the Phigdet object for the given PhidgetId, or - if no
//     such Phidget exists, return with an given error code.
//
// Parameters:
// 
//     id - PhidgetId of the Phidget to be returned. This id is provided
//         upon object creation.
//
//     pPhidget - variable where the pointer to the phidget object is
//         stored (See the below example). This value will be compared
//         to NULL. If it is NULL after getting the pointer, the routine
//         will exit with the given return value from the current function.
//         The value is unchanged in this routine.
//
//     ret - If no phidget with the given ID exists, we will exit the
//         function where this routine is called with ret as return value.
//
// Example of usage:
//
//     PhidgetIfKit* pPhidget =
//         (PhidgetIfKit*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);
//
#define phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, ret) \
	phidgetPlugin_GetPhidgetFromId(id); \
	if(pPhidget == NULL) \
		return ret;


//////////////////////////////////////////////////////////////////////////
// Plugin Functions Declaration

// Plugin initialization / shutdown
void phidgetPlugin_Initialise();
void phidgetPlugin_Shutdown();

// managing global Phigdets list
PhidgetId phidgetPlugin_GetNextPhidgetId();
BOOL phidgetPlugin_AddPhidget(Phidget* pPhidget);
BOOL phidgetPlugin_RemovePhidget(Phidget* pPhidget);
Phidget* phidgetPlugin_GetPhidgetFromId(PhidgetId id);

// string helper functions
char* phidgetPlugin_ConvertToCString(char* pcString, int iLength);
void phidgetPlugin_FreeCString(char* pszString);


#endif //_PHIDGETPLUGIN_H
